import logging
import sys

from loguru import logger as base_logger

logger = base_logger


def init_logger():
    global logger
    logging.disable(logging.CRITICAL)
    logger.remove()
    logger.add(
        sink=sys.stdout,
        level=logging.INFO,
        format="<green>{time:YYYY-MM-DD HH:mm:ss}</green> | "
        "<level>{level}</level> | "
        "<level>{message}</level>",
    )



from typing import List

from pydantic import BaseModel, HttpUrl


class AuthorInfo(BaseModel):
    id: str
    username: str
    nickname: str
    avatar: HttpUrl
    signature: str
    is_verified: bool

    class Config:
        fields = {
            "username": "uniqueId",
            "avatar": "avatarLarger",
            "is_verified": "verified",
        }


class MusicInfo(BaseModel):
    id: str
    title: str
    link: str = ""
    author_name: str = ""
    is_original: bool
    cover: str = ""

    class Config:
        fields = {
            "is_original": "original",
            "author_name": "authorName",
            "link": "playUrl",
            "cover": "coverLarge",
        }


class StatisticsInfo(BaseModel):
    plays: int
    likes: int
    comments: int
    shares: int

    class Config:
        fields = {
            "likes": "diggCount",
            "shares": "shareCount",
            "comments": "commentCount",
            "plays": "playCount",
        }


class ChallengeInfo(BaseModel):
    id: str
    title: str
    desc: str
    profile_thumb: str
    profile_medium: str
    profile_larger: str
    cover_thumb: str = ""
    cover_medium: str = ""
    cover_larger: str = ""

    class Config:
        fields = {
            "profile_thumb": "profileThumb",
            "profile_medium": "profileMedium",
            "profile_larger": "profileLarger",
            "cover_thumb": "coverThumb",
            "cover_medium": "coverMedium",
            "cover_larger": "coverLarger",
        }


class VideoInfo(BaseModel):
    id: str
    height: int
    width: int
    duration: int
    ratio: str
    cover: HttpUrl
    play_addr: HttpUrl
    download_addr: HttpUrl

    class Config:
        fields = {
            "play_addr": "playAddr",
            "download_addr": "downloadAddr",
        }

    @property
    def original_video_url(self) -> str:
        return f"https://api2.musical.ly/aweme/v1/playwm/?video_id={self.id}"


class FeedItem(BaseModel):
    id: str
    desc: str
    create_time: int
    author: AuthorInfo
    music: MusicInfo
    stats: StatisticsInfo
    video: VideoInfo
    challenges: List[ChallengeInfo] = []

    class Config:
        fields = {
            "create_time": "createTime",
        }


class FeedItems(BaseModel):
    __root__: List[FeedItem]



import asyncio
import json
from pathlib import Path
from typing import List, Optional
from urllib.parse import urlencode, urljoin

from dynaconf import settings
from pyppeteer import launch
from pyppeteer.browser import Browser
from pyppeteer.page import Page, Response
from pyppeteer_stealth import (
    console_debug,
    iframe_content_window,
    media_codecs,
    navigator_permissions,
    navigator_plugins,
    navigator_webdriver,
    user_agent,
    webgl_vendor,
    window_outerdimensions,
)

from tiktokpy.utils.client import block_resources_and_sentry
from tiktokpy.utils.logger import logger


class Client:
    def __init__(self):
        self.base_url = settings.BASE_URL

        self.cookies = json.loads(settings.get("COOKIES", "[]"))

    async def init_browser(self, headless: bool):
        params = {
            "headless": headless,
            "args": [
                "--no-sandbox",
                "--disable-setuid-sandbox",
                "--disable-dev-shm-usage",
                "--disable-web-security",
            ],
        }

        self.browser: Browser = await launch(**params)
        logger.debug(f"🎉 Browser launched. Options: {params}")

    async def stealth(self, page: Page):
        await iframe_content_window(page)
        await console_debug(page)
        await media_codecs(page)
        await navigator_permissions(page)
        await navigator_plugins(page)
        await navigator_webdriver(page)
        await user_agent(page)
        await webgl_vendor(page)
        await window_outerdimensions(page)

    async def new_page(self, blocked_resources: Optional[List[str]] = None) -> Page:
        page: Page = await self.browser.newPage()

        # set stealth mode for tiktok
        await self.stealth(page)

        await page.setCookie(*self.cookies)

        if blocked_resources is not None:
            await page.setRequestInterception(True)
            page.on(
                "request",
                lambda req: asyncio.create_task(
                    block_resources_and_sentry(req, blocked_resources),
                ),
            )

        return page

    async def goto(
        self,
        url: str,
        page: Page,
        query_params: Optional[dict] = None,
        *args,
        **kwargs,
    ) -> Response:
        full_url = urljoin(self.base_url, url)

        if query_params:
            query_params = urlencode(query=query_params)
            full_url = f"{full_url}?{query_params}"

        return await page.goto(full_url, *args, **kwargs)

    async def screenshot(self, path: str, page: Page):
        Path(path).parent.mkdir(parents=True, exist_ok=True)

        await page.screenshot({"path": path})

    @classmethod
    async def create(cls, headless: bool = True):
        self = Client()
        await self.init_browser(headless=headless)

        return self



import typing
from datetime import datetime
from types import TracebackType
from typing import List, Optional

import humanize
from dynaconf import settings

from tiktokpy.client import Client
from tiktokpy.client.login import Login
from tiktokpy.client.trending import Trending
from tiktokpy.client.user import User
from tiktokpy.models.feed import FeedItem, FeedItems
from tiktokpy.utils.logger import init_logger, logger
from tiktokpy.utils.settings import load_or_create_settings


class TikTokPy:
    def __init__(self, settings_path: Optional[str] = None):
        from tiktokpy import __version__

        init_logger()
        self.started_at = datetime.now()

        logger.info("🥳 TikTokPy initialized. Version: {}", __version__)

        load_or_create_settings(path=settings_path)

        if settings.get("COOKIES") and settings.get("USERNAME"):
            logger.info(f"✅ Used cookies of @{settings.USERNAME}")
        else:
            logger.info("🛑 Cookies not found, anonymous mode")

    async def __aenter__(self):
        await self._init_bot()

        return self

    async def __aexit__(
        self,
        exc_type: typing.Type[BaseException] = None,
        exc_value: BaseException = None,
        traceback: TracebackType = None,
    ) -> None:
        logger.debug("🤔Trying to close browser..")

        await self.client.browser.close()

        logger.debug("✋ Browser successfully closed")
        logger.info(
            "✋ TikTokPy finished working. Session lasted: {}",
            humanize.naturaldelta(datetime.now() - self.started_at),
        )

    async def trending(self, amount: int = 50, lang: str = "en") -> List[FeedItem]:
        logger.info("📈 Getting trending items")
        items = await Trending(client=self.client).feed(amount=amount, lang=lang)

        logger.info(f"📹 Found {len(items)} videos")
        _trending = FeedItems(__root__=items)

        return _trending.__root__

    async def follow(self, username: str):
        username = f"@{username.lstrip('@')}"
        await User(client=self.client).follow(username=username)

    async def like(self, feed_item: FeedItem):
        await User(client=self.client).like(
            username=feed_item.author.username,
            video_id=feed_item.id,
        )

    async def unlike(self, feed_item: FeedItem):
        await User(client=self.client).unlike(
            username=feed_item.author.username,
            video_id=feed_item.id,
        )

    async def unfollow(self, username: str):
        username = f"@{username.lstrip('@')}"
        await User(client=self.client).unfollow(username=username)

    async def login_session(self):
        await Login().manual_login()

    async def user_feed(self, username: str, amount: int = 50) -> List[FeedItem]:
        username = f"@{username.lstrip('@')}"
        logger.info(f"📈 Getting {username} feed")
        items = await User(client=self.client).feed(username=username, amount=amount)

        logger.info(f"📹 Found {len(items)} videos")
        feed = FeedItems(__root__=items)

        return feed.__root__

    async def _init_bot(self):
        self.client: Client = await Client.create()

    @classmethod
    async def create(cls):
        self = TikTokPy()
        await self._init_bot()

        return self

    async def screenshot(self, page, name=""):
        filename = f"{name}_{datetime.now()}".lstrip("_")

        await self.client.screenshot(
            path=f"{settings.HOME_DIR}/screenshots/{filename}.png",
            page=page,
        )



from typing import List, Optional

from pydantic import BaseModel


class UserInfo(BaseModel):
    username: str
    nickname: str


class MusicInfo(BaseModel):
    title: str
    link: str


class StatisticsInfo(BaseModel):
    likes: int
    comments: int
    shares: int


class VideoInfo(BaseModel):
    id: str
    # link on page
    link: str
    video_url: Optional[str]
    # preview_image: str

    # @property
    # def original_video_url(self) -> str:
    #     return f"https://api2.musical.ly/aweme/v1/playwm/?video_id={self.private_id}"


class FeedItem(BaseModel):
    user_info: UserInfo
    avatar: str
    title: str
    music_info: MusicInfo
    statistics: StatisticsInfo
    video_info: VideoInfo


class FeedItems(BaseModel):
    __root__: List[FeedItem]



import asyncio
from typing import List

from pyppeteer.page import Page
from tqdm import tqdm

from tiktokpy.client import Client
from tiktokpy.utils.client import catch_response_and_store, catch_response_info
from tiktokpy.utils.logger import logger


class User:
    def __init__(self, client: Client):
        self.client = client

    async def like(self, username: str, video_id: str):
        page: Page = await self.client.new_page(blocked_resources=["image", "media", "font"])
        logger.debug(f"👥 Like video id {video_id} of @{username}")

        like_info_queue: asyncio.Queue = asyncio.Queue(maxsize=1)
        video_info_queue: asyncio.Queue = asyncio.Queue(maxsize=1)

        page.on(
            "response",
            lambda res: asyncio.create_task(
                catch_response_info(res, video_info_queue, "/item/detail"),
            ),
        )

        page.on(
            "response",
            lambda res: asyncio.create_task(
                catch_response_info(res, like_info_queue, "/commit/item/digg"),
            ),
        )

        logger.info(f"🧭 Going to @{username}'s video {video_id} page for like")

        await self.client.goto(
            f"/@{username}/video/{video_id}",
            page=page,
            options={"waitUntil": "networkidle0"},
        )

        video_info = await video_info_queue.get()

        if video_info["itemInfo"]["itemStruct"]["digged"]:
            logger.info(f"😏 @{username}'s video {video_id} already liked")
            return

        like_part = await page.J(".like-part")

        if like_part:
            await page.click(".like-part")
        else:
            await page.click(
                ".video-feed-container .lazyload-wrapper:first-child .bar-item-wrapper:first-child",
            )

        like_info = await like_info_queue.get()

        if like_info["status_code"] == 0:
            logger.info(f"👍 @{username}'s video {video_id} liked")
        else:
            logger.warning(f"⚠️  @{username}'s video {video_id} probably not liked")

        await page.close()

    async def unlike(self, username: str, video_id: str):
        page: Page = await self.client.new_page(blocked_resources=["image", "media", "font"])
        logger.debug(f"👥 Unlike video id {video_id} of @{username}")

        like_info_queue: asyncio.Queue = asyncio.Queue(maxsize=1)
        video_info_queue: asyncio.Queue = asyncio.Queue(maxsize=1)

        page.on(
            "response",
            lambda res: asyncio.create_task(
                catch_response_info(res, video_info_queue, "/item/detail"),
            ),
        )

        page.on(
            "response",
            lambda res: asyncio.create_task(
                catch_response_info(res, like_info_queue, "/commit/item/digg"),
            ),
        )

        logger.info(f"🧭 Going to @{username}'s video {video_id} page for unlike")

        await self.client.goto(
            f"/@{username}/video/{video_id}",
            page=page,
            options={"waitUntil": "networkidle0"},
        )

        video_info = await video_info_queue.get()

        if not video_info["itemInfo"]["itemStruct"]["digged"]:
            logger.info(f"😏 @{username}'s video {video_id} already unliked")
            return

        like_part = await page.J(".like-part")

        if like_part:
            await page.click(".like-part")
        else:
            await page.click(
                ".video-feed-container .lazyload-wrapper:first-child .bar-item-wrapper:first-child",
            )

        like_info = await like_info_queue.get()

        if like_info["status_code"] == 0:
            logger.info(f"👎 @{username}'s video {video_id} unliked")
        else:
            logger.warning(f"⚠️  @{username}'s video {video_id} probably not unliked")

        await page.close()

    async def follow(self, username: str):
        page: Page = await self.client.new_page(blocked_resources=["image", "media", "font"])
        logger.debug(f"👥 Follow {username}")

        follow_info_queue: asyncio.Queue = asyncio.Queue(maxsize=1)

        page.on(
            "response",
            lambda res: asyncio.create_task(
                catch_response_info(res, follow_info_queue, "/commit/follow/user"),
            ),
        )

        logger.info(f"🧭 Going to {username}'s page for following")

        await self.client.goto(
            f"/@{username.lstrip('@')}",
            page=page,
            options={"waitUntil": "networkidle0"},
        )

        follow_title: str = await page.Jeval(
            ".follow-button",
            pageFunction="element => element.textContent",
        )

        if follow_title.lower() != "follow":
            logger.info(f"😏 {username} already followed")
            return

        await page.click(".follow-button")

        follow_info = await follow_info_queue.get()

        if follow_info["status_code"] == 0:
            logger.info(f"➕ {username} followed")
        else:
            logger.warning(f"⚠️  {username} probably not followed")

        await page.close()

    async def unfollow(self, username: str):
        page: Page = await self.client.new_page(blocked_resources=["image", "media", "font"])
        logger.debug(f"👥 Unfollow {username}")

        unfollow_info_queue: asyncio.Queue = asyncio.Queue(maxsize=1)

        page.on(
            "response",
            lambda res: asyncio.create_task(
                catch_response_info(res, unfollow_info_queue, "/commit/follow/user"),
            ),
        )

        logger.info(f"🧭 Going to {username}'s page for unfollowing")

        await self.client.goto(
            f"/@{username.lstrip('@')}",
            page=page,
            options={"waitUntil": "networkidle0"},
        )

        follow_title: str = await page.Jeval(
            ".follow-button",
            pageFunction="element => element.textContent",
        )

        if follow_title.lower() != "following":
            logger.info(f"😏 {username} already unfollowed")
            return

        await page.click(".follow-button")

        unfollow_info = await unfollow_info_queue.get()

        if unfollow_info["status_code"] == 0:
            logger.info(f"➖ {username} unfollowed")
        else:
            logger.warning(f"⚠️  {username} probably not unfollowed")

        await page.close()

    async def feed(self, username: str, amount: int):
        page: Page = await self.client.new_page(blocked_resources=["image", "media", "font"])
        logger.debug(f"📨 Request {username} feed")

        result: List[dict] = []
        user_info_queue: asyncio.Queue = asyncio.Queue(maxsize=1)

        page.on(
            "response",
            lambda res: asyncio.create_task(catch_response_and_store(res, result)),
        )

        page.on(
            "response",
            lambda res: asyncio.create_task(
                catch_response_info(res, user_info_queue, "/user/detail"),
            ),
        )
        _ = await self.client.goto(f"/{username}", page=page, options={"waitUntil": "networkidle0"})
        logger.debug(f"📭 Got {username} feed")

        await page.waitForSelector(".video-feed-item", options={"visible": True})

        user_info = await user_info_queue.get()
        user_video_count = user_info["userInfo"]["stats"]["videoCount"]

        if user_video_count < amount:
            logger.info(
                f"⚠️  User {username} has only {user_video_count} videos. "
                f"Set amount from {amount} to {user_video_count}",
            )
            amount = user_video_count

        pbar = tqdm(total=amount, desc=f"📈 Getting {username} feed")
        pbar.n = min(len(result), amount)
        pbar.refresh()

        attempts = 0
        last_result = len(result)

        while len(result) < amount:
            logger.debug("🖱 Trying to scroll to last video item")
            await page.evaluate(
                """
                document.querySelector('.video-feed-item:last-child')
                    .scrollIntoView();
            """,
            )
            await page.waitFor(1_000)

            elements = await page.JJ(".video-feed-item")
            logger.debug(f"🔎 Found {len(elements)} items for clear")

            pbar.n = min(len(result), amount)
            pbar.refresh()

            if last_result == len(result):
                attempts += 1
            else:
                attempts = 0

            if attempts > 10:
                pbar.clear()
                pbar.total = len(result)
                logger.info(
                    f"⚠️  After 10 attempts found {len(result)} videos. "
                    f"Probably some videos are private",
                )
                break

            last_result = len(result)

            if len(elements) < 500:
                logger.debug("🔻 Too less for clearing page")
                continue

            await page.JJeval(
                ".video-feed-item:not(:last-child)",
                pageFunction="(elements) => elements.forEach(el => el.remove())",
            )
            logger.debug(f"🎉 Cleaned {len(elements) - 1} items from page")
            await page.waitFor(30_000)

        await page.close()
        pbar.close()
        return result[:amount]



import json

from dynaconf import loaders, settings

from tiktokpy.client import Client
from tiktokpy.utils.logger import logger
from tiktokpy.utils.settings import BASE_SETTINGS


class Login:
    async def manual_login(self):
        client = await Client.create(headless=False)
        page = await client.new_page()

        await client.stealth(page)

        await client.goto("/login", page)
        await page.waitForSelector(".profile", options={"timeout": 0})
        await page.hover(".profile")

        await page.waitFor(".profile-actions > li:first-child")
        # going to "View profile" page
        await page.click(".profile-actions > li:first-child")
        await page.waitForSelector(".share-title", options={"timeout": 0})

        username = await page.Jeval(".share-title", pageFunction="element => element.textContent")
        username = username.strip()

        sub_title = await page.Jeval(
            ".share-sub-title",
            pageFunction="element => element.textContent",
        )

        logger.info(f"🔑 Logged as @{username} aka {sub_title}")

        cookies = await page.cookies()

        loaders.write(
            f"{settings.HOME_DIR}/settings.toml",
            {**BASE_SETTINGS, **{"COOKIES": json.dumps(cookies), "USERNAME": username}},
            env="default",
        )

        await client.browser.close()



import typer

from tiktokpy import TikTokPy
from tiktokpy.cli.utils import coro

app = typer.Typer()


@app.command()
@coro
async def login():
    async with TikTokPy() as bot:
        await bot.login_session()


@app.callback()
def callback():
    pass



import asyncio
from typing import List

from tqdm import tqdm

from tiktokpy.client import Client
from tiktokpy.utils.client import catch_response_and_store
from tiktokpy.utils.logger import logger


class Trending:
    def __init__(self, client: Client):
        self.client = client

    async def feed(self, amount: int, lang: str = "en"):
        page = await self.client.new_page(blocked_resources=["media", "image", "font"])

        logger.debug('📨 Request "Trending" page')

        result: List[dict] = []

        page.on(
            "response",
            lambda res: asyncio.create_task(catch_response_and_store(res, result)),
        )
        _ = await self.client.goto(
            "/foryou",
            query_params={"lang": lang},
            page=page,
            options={"waitUntil": "networkidle0"},
        )
        logger.debug('📭 Got response from "Trending" page')

        pbar = tqdm(total=amount, desc=f"📈 Getting trending {lang.upper()}")
        pbar.n = min(len(result), amount)
        pbar.refresh()

        while len(result) < amount:

            logger.debug("🖱 Trying to scroll to last video item")
            await page.evaluate(
                """
                document.querySelector('.video-feed-container > .lazyload-wrapper:last-child')
                    .scrollIntoView();
            """,
            )
            await page.waitFor(1_000)

            elements = await page.JJ(".video-feed-item")
            logger.debug(f"🔎 Found {len(elements)} items for clear")

            pbar.n = min(len(result), amount)
            pbar.refresh()

            if len(elements) < 500:
                logger.debug("🔻 Too less for clearing page")
                continue

            await page.JJeval(
                ".video-feed-container > .lazyload-wrapper:not(:last-child)",
                pageFunction="(elements) => elements.forEach(el => el.remove())",
            )
            logger.debug(f"🎉 Cleaned {len(elements) - 1} items from page")
            await page.waitFor(30_000)

        await page.close()
        pbar.close()
        return result[:amount]



from typing import List

from tiktokpy.utils.logger import logger


async def block_resources_and_sentry(request, types: List[str]):
    is_blocked = False

    if request.resourceType in types:
        is_blocked = True

    if "/sentry/" in request.url:
        is_blocked = True

    if is_blocked:
        await request.abort()
    else:
        await request.continue_()


async def catch_response_and_store(response, result):
    if "/item_list" in response.url:
        logger.debug(response.url)
        data = await response.json()

        for item in data["items"]:
            result.append(item)
        logger.debug(f"🛒 Collected {len(data['items'])} items. Total: {len(result)}")


async def catch_response_info(response, queue, url: str):
    if url in response.url:
        logger.debug(response.url)
        result = await response.json()

        await queue.put(result)
        logger.debug(f"🛒 Collected response: {result}")



from pathlib import Path
from typing import Optional

from dynaconf import loaders, settings

from tiktokpy.utils.logger import logger

DEFAULT_PATH = "settings.toml"
BASE_SETTINGS = {
    "BASE_URL": "https://www.tiktok.com/",
    "HEADLESS": True,
    "LANG": "en",
    "HOME_DIR": str(Path().absolute()),
}


def load_or_create_settings(path: Optional[str]):
    path = path or DEFAULT_PATH

    if not Path(path).exists():
        default_settings_path = str(Path.cwd() / Path(DEFAULT_PATH))
        logger.info(
            f'🔧 Settings in path directory not found "{Path(path).absolute()}". '
            f"I'll create default settings here: {default_settings_path}",
        )
        Path(path).parent.mkdir(parents=True, exist_ok=True)

        loaders.write(default_settings_path, BASE_SETTINGS, env="default")

    settings.load_file(path=path)

    logger.info("🔧 Settings successfully loaded")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tiktokpy',
 'tiktokpy.bot',
 'tiktokpy.cli',
 'tiktokpy.client',
 'tiktokpy.models',
 'tiktokpy.models.html',
 'tiktokpy.parsers',
 'tiktokpy.utils']

package_data = \
{'': ['*']}

install_requires = \
['dynaconf>=3.0.0,<4.0.0',
 'humanize>=2.5.0,<3.0.0',
 'loguru>=0.5.0,<0.6.0',
 'pydantic>=1.6.1,<2.0.0',
 'pyppeteer-stealth>=1.0.1,<2.0.0',
 'pyppeteer>=0.2.2,<0.3.0',
 'tqdm>=4.48.0,<5.0.0',
 'typer>=0.3.1,<0.4.0']

extras_require = \
{'html': ['parsel>=1.6.0,<2.0.0']}

entry_points = \
{'console_scripts': ['tiktokpy = tiktokpy.cli:app']}

setup_kwargs = {
    'name': 'tiktokpy',
    'version': '0.8.6',
    'description': 'Tool for automated TikTok interactions.',
    'long_description': '<h1 align="center" style="font-size: 3rem;">\nTikTokPy\n</h1>\n<p align="center">\n <em>Tooling that <b>automates</b> your social media interactions to “farm” Likes and Followers on TikTok</em></p>\n\n<p align="center">\n<a href="https://travis-ci.org/sudoguy/tiktokpy">\n    <img src="https://travis-ci.org/sudoguy/tiktokpy.svg?branch=master" alt="Build Status">\n</a>\n<a href="https://pypi.org/project/tiktokpy/">\n    <img src="https://badge.fury.io/py/tiktokpy.svg" alt="Package version">\n</a>\n</p>\n\n---\n\n## Quickstart\n\n```python\nimport asyncio\nfrom tiktokpy import TikTokPy\n\n\nasync def main():\n    async with TikTokPy() as bot:\n        # Do you want to get trending videos? You can!\n        trending_items = await bot.trending(amount=5)\n\n        for item in trending_items:\n            # ❤️ you can like videos\n            await bot.like(item)\n            # or unlike them\n            await bot.unlike(item)\n            # or follow users\n            await bot.follow(item.author.username)\n            # as and unfollow\n            await bot.unfollow(item.author.username)\n\n        # 😏 getting user\'s feed\n        user_feed_items = await bot.user_feed(username="justinbieber", amount=5)\n\n        for item in user_feed_items:\n            # 🎧 get music title, cover, link, author name..\n            print("Music title: ", item.music.title)\n            # #️⃣ print all tag\'s title of video\n            print([tag.title for tag in item.challenges])\n            # 📈 check all video stats\n            print("Comments: ", item.stats.comments)\n            print("Plays: ", item.stats.plays)\n            print("Shares: ", item.stats.shares)\n            print("Likes: ", item.stats.likes)\n\n        # and many other things 😉\n\n\nasyncio.run(main())\n```\n\n## Installation\n\nInstall with pip:\n\n```shell\npip install tiktokpy\n```\n',
    'author': 'Evgeny Kemerov',
    'author_email': 'eskemerov@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/sudoguy/tiktokpy/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, List, Optional, Union

from parsel import Selector

from tiktokpy.utils.logger import logger


class FeedItemsParser:
    def __init__(self, htmls: List[str]):
        self.htmls = htmls

    def _parse_actions(self, action_value: str) -> int:
        if "M" in action_value:
            # Example: 5.1M
            # Remove "M" from value and cast to float
            new_value = float(action_value[:-1])
            new_value = int(new_value * 1_000_000)
        elif "K" in action_value:
            # Example: 5.1K
            # Remove "K" from value and cast to float
            new_value = float(action_value[:-1])
            new_value = int(new_value * 1_000)
        else:
            new_value = int(action_value)

        return new_value

    def loads(self) -> List[Dict[str, Any]]:
        result = []

        for html in self.htmls:
            self.selector = Selector(text=html, type="html", base_url="https://www.tiktok.com/")

            body = {
                "avatar": self.avatar(),
                "title": self.title(),
                "user_info": self.user_info(),
                "music_info": self.music_info(),
                "statistics": self.statistics(),
                "video_info": self.video_info(),
            }

            result.append(body)
        logger.debug(f"Parsed {len(self.htmls)} trending items")

        return result

    def avatar(self) -> str:
        img = self.selector.css(".avatar > img")
        if not img:
            logger.warning("Avatar src not found")
            return ""

        return img.attrib["src"]

    def statistics(self) -> Dict[str, int]:
        likes = self.selector.css('.pc-action-bar strong[title="like"]::text').get(0)
        comments = self.selector.css('.pc-action-bar strong[title="comment"]::text').get(0)
        shares = self.selector.css('.pc-action-bar strong[title="share"]::text').get(0)

        return {
            "likes": self._parse_actions(likes),
            "comments": self._parse_actions(comments),
            "shares": self._parse_actions(shares),
        }

    def video_info(self) -> Dict[str, Optional[str]]:
        link = self.selector.css("a.item-video-card-wrapper").attrib["href"]
        _id = link.split("/")[-1]
        video_url = self.selector.css(".video-player")
        if video_url:
            video_url = video_url.attrib["src"]

        return {
            "id": _id,
            "link": link,
            "video_url": video_url or None,
        }

    def title(self) -> str:
        title = self.selector.css(".item-meta-title > strong::text").get("")

        return title

    def music_info(self) -> Dict[str, str]:
        title = self.selector.css(".music-title-content::text").get("")
        link = self.selector.css(".music-info a").attrib["href"]

        return {
            "title": title,
            "link": link,
        }

    def user_info(self) -> Dict[str, Union[str, bool]]:
        username = self.selector.css(".user-username::text").get("")
        nickname = self.selector.css(".user-nickname::text").get("")
        is_verified = bool(self.selector.css(".verified"))

        return {
            "username": username,
            "nickname": nickname,
            "is_verified": is_verified,
        }
